#####################################################################################################################
# Licensed Materials - Property of IBM
# Rational ClearQuest
# (C) Copyright IBM Corp. 2003-2011.  All Rights Reserved.
# US Government Users Restricted Rights -
# Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
#
# Update note: This script currently has an optional parameter for the build start time.  At this time, the record
# cannot be updated with this information.  If the parameter is used and a value passed in, the record
# is not updated with the information. 
######################################################################################################################
# 
# 
# Updated by: Shirley Dango
# Updated on: April 21, 2003
# Updated January 2008, by Katie Carpenter
#
#This script utilized the ClearQuest COM API and the ClearCase cleartool commands.
# 
# Variable used in this script are as followings:
#	user			The login id of the ClearQuest user
#	pw			The password of the ClearQuest user
#	dbname			The schema repository connection to the database
#	dbset			The user database connection
#	pvob			The name of the project vob
#	baseline		The name of the baseline
#	id			The id of the BTBuild record to update
#	buildstatus		Specifies what the status of the build.  If it passed or failed.
#	buildstarttime		Time of day build was started.
#	buildendtime		Time of day build completed.
#	logfile			The path where the log file will be sent
#	test			This is a flag that can be set for testing file inputs
#	help			This is a flag for the help
#	EDIT_BUILD_ACTION	This variable will set a session variable for ClearQuest access.
#######################################################################################################################
#
# Change Log:
# Flag      Item                Date        Programmer         Comments
# thl1   RATLC01087702       06/28/2005      THL               UCM Integration Scripts should use ratlperl not cqperl
# thl2   RATLC00475437       10/18/2005      THL               the baseline create perl module needs to also check the returned 
#                                                              status from CQ API calls.
#        RATLC00396394       1/14/2008       katherns          Update scripts for M3 release.
#
#######################################################################################################################
#!ratlperl -l -w
# Changed above line to read ratlperl from cqperl thl1
use strict; 
use vars qw($Bin);
use FindBin qw($Bin);
use lib "$Bin";
use Getopt::Long;
use File::Basename;
use Carp;
use FileHandle;
use File::Spec;  
use CQPerlExt;
use Env;
use IO::File;
use LWP::Simple;
use POSIX;
use Time::Local;
# use Date::Manip qw(ParseDate UnixDate);

my $progname = basename $0;
my ($entity);
my ($failure,$status); #thl2
my $CRList;
my $item;
my $slashvob;
my $noslashvob;
my $buildtype;
my $securitypolicy;

# Unbuffer STDERR and STDOUT
select STDERR;  
$| = 1;
select STDOUT;
$| = 1;

#Define Logger parameters
use Logger qw(CONSOLE_ON CONSOLE_OFF CONSOLE_TERSE);
my $logger = Logger->new(); # the logger element

# Create the utility object
use UtilLib;
my $ut = UtilLib->new();
my $log_time = $ut->ut_file_name_date_time_stamp();

my $result = &main();

if ($result) {
    exit(0); # DOS Success Code
} else {
    exit(1); # DOS Error Code
}

####
# Subroutine to display the usage
sub Usage {
    print "

    CQ API gpăOIAZbV쐬郆[eBeB
    gp@:
        $progname
 	-id RATLC000xxxxx
 	-user <[U[>
 	-pw <pX[h>
 	-dbname <f[^x[X>
 	-dbset <f[^x[X Zbg>
 	-pvob<pvob>
 	-baseline<x[XC>
	-buildstatus<rh Xe[^X>
	-securitypolicy<ALM ZLeB |V[>
	-buildtype<rh ^Cv>
    -buildstarttime<rhJn>
    -buildendtime<rhI>
    -logfile
    -test
    -help		
";
return 0;
}

sub main {	
####
# Define the input parameters
my @opts = qw(
    id=s
    user=s	
    pw=s		
    dbname=s
    dbset=s
    pvob=s
    baseline=s
    buildstatus:s
    securitypolicy=s
    buildtype=s
    buildstarttime:s
    buildendtime:s
    state=s
    logfile=s
    test
    help
);

my %opts = (
    id => undef,
    user => undef, 
    pw => undef,
    dbname => undef,
    dbset => undef,
    pvob => undef,
    baseline => undef,
    securitypolicy=> undef,
    buildtype=> undef,
    logfile =>undef,
    state => "Modify", 
);

####
# Get the input parameters
unless (GetOptions(\%opts, @opts)) {
    $logger->log_print_message(__LINE__ . "R}hsւ̈p@:");	
     my $retval = &Usage;
     return 0;
}

#Define the LogFile
if ($opts{logfile}) {
	my $retval;
	my $logdir = dirname($opts{logfile});
	if ( ! -e $logdir) {
		$retval = $ut->ut_create_directory_tree($logdir);
		if ( ! $retval) {
            		$logger->log_print_error(__LINE__ . "O t@C fBNg $logdir 쐬ł܂ł");
            		$logger->log_print_error(__LINE__ . "XNvg̎śAȂŌp܂B");
        	}
	}
	$retval = $logger->log_set_file_on($opts{logfile});
	if ( ! $retval) {
        	$logger->log_print_error(__LINE__ . "O t@C $opts{logfile} J܂łB");
        	$logger->log_print_error(__LINE__ . "XNvg̎śAȂŌp܂B");
   	 }
        $logger->log_set_console(CONSOLE_OFF);
	$logger->log_print_message("MO $log_time ɊJn܂B"); 
}	 

####
# Display the help usage
if ($opts{help}) {
 
   	print "Kvȃp[^ɃXgĂ܂: \n";     
        print "-user		ClearQuest [U[̃OC ID\n";
	print "-pw		ClearQuest [U[̃pX[h\n";
        print "-dbname		XL[} |Wg̃f[^x[Xւ̐ڑ\n";
	print "-dbset		[U[ f[^x[Xڑ\n";
	print "-logfile		O t@CMpX\n";
        print "-pvob		vWFNg VOB ̖O\n";
	print "-baseline  	x[XC̖O\n";
	print "-securitypolicy	R[hɊ蓖Ă ALM ZLeB |V[\n";
	print "-buildtype  	rh̃^Cv\n";
	print "IvV p[^ɃXgĂ܂: \n";
	print "-state <state>   rh R[hv[g\n";	
	print "-buildstatus		rh̃Xe[^Xw肵܂B󂯓n邩sꍇB\n";
	print "-buildstarttime		rhJnꂽB\n";
	print "-buildendtime		rhB\n";
	print "-test		́AeXg t@C͂ɐݒłIvV tOłB\n";
	print "-help            ́AwṽIvV tOłB\n";
    return 1;
}

# Display test inputs to log file 
if ($opts{test}){
$logger->log_print_message(__LINE__ . "`ꂽ[U[:");
	$logger->log_print_message(__LINE__ . " `ꂽrh R[h ID $opts{id}B");
	$logger->log_print_message(__LINE__ . "`ꂽ[U[ OC $opts{user} łB");
	$logger->log_print_message(__LINE__ . "`ꂽ dbset  $opts{dbset} łB");
	$logger->log_print_message(__LINE__ . "`ꂽf[^x[X $opts{dbname} łB");
	$logger->log_print_message(__LINE__ . "`ꂽx[XC $opts{baseline} łB");
	$logger->log_print_message(__LINE__ . " `ꂽԂ $opts{state} łB");
	$logger->log_print_message(__LINE__ . "`ꂽvWFNg VOB  $opts{pvob} łB");
	$logger->log_print_message(__LINE__ . " `ꂽrh ^Cv $opts{buildtype} łB");
	$logger->log_print_message(__LINE__ . " `ꂽi $opts{product} łB");
	$logger->log_print_message(__LINE__ . " `ꂽZLeB |V[ $opts{securitypolicy} łB");
	if ($opts{logfile}){ $logger->log_print_message("`ꂽO t@C $opts{logfile} łB");}
	if ($opts{buildstatus}){ $logger->log_print_message("`ꂽrhԂ $opts{buildstatus} łB");}
	if ($opts{buildstarttime}){ $logger->log_print_message("`ꂽrhJnԂ $opts{buildstarttime} łB");}
	if ($opts{buildendtime}){ $logger->log_print_message("`ꂽrhIԂ $opts{buildendtime} łB");}
	return 1;
}
####
# End program if any of the required parameters are not defined


if (!defined $opts{user}
  or !defined $opts{pw} 
  or !defined $opts{dbname} 
  or !defined $opts{dbset} 
  or !defined $opts{id}
  or !defined $opts{pvob} 
  or !defined $opts{securitypolicy}
  or !defined $opts{buildtype}) {
  $logger->log_print_message(__LINE__ . "R}hsւ̈p@:");
  my $retval = &Usage();
  return 0;
}
;
####
# Start a CQ session
print ("Starting CQ Session\n") ;
my $session = CQPerlExt::CQSession_Build();
####
# Set session variable 
#$session->SetNameValue("EDIT_BUILD_ACTION", "ACCESS_OK");
# Logon to the specified database

eval {$session->UserLogon("$opts{user}", "$opts{pw}", "$opts{dbname}", "$opts{dbset}")};

if ($@) {
$logger->log_print_error(__LINE__ . "main: $opts{dbname} ւ̃OCɎs܂: $@"); 
return 0;
}

$logger->log_print_message(__LINE__ . "$opts{dbname} ւ̃OC܂B");


#### Set up vobs
##Passing in the vob to CQ requires no slashes on the vob, passing in to CC requires a slash on the vob
sub slash_noslash {
	my $pvob = shift;
	my $slashvob;
	my $noslashvob;
	if ($pvob !~ /[\/\\]/) {
	  $slashvob = '\\' .$pvob;
	  $noslashvob = $pvob;
	}
	else{
	  $slashvob = $pvob;
	  $noslashvob = $pvob ;
	  $noslashvob=~ s/\\//g; 
	}
	return ($slashvob, $noslashvob);
}
($slashvob, $noslashvob ) = &slash_noslash($opts{pvob});

#### Build a query to find the Build record based upon Build ID
my $record;
eval {$record = $session->GetEntity("BTBuild", $opts{id})};
if ($@) {
	$logger->log_print_error(__LINE__ . "main: rh R[h ID $opts{id} Ɉv郌R[h܂ł"); 
	$logger->log_print_error(__LINE__ . "main: create_build_record.pl sāAKvȃrh R[h쐬܂B"); 
		return 0;
}
	{
	#### Print out the resultset
	$logger->log_print_message(__LINE__ . " R[h $opts{id} ܂");
	
	####Get record 
	# Add updates to the record
	$session->EditEntity($record,"$opts{state}");
	$logger->log_print_message(__LINE__ . " $opts{state} R[h: $opts{id}");
	 	
        #Set required values

	my $baselinename = $noslashvob . ' ' . $opts{baseline};
	
	$record->SetFieldValue("ALMBaseline", $baselinename);
		$logger->log_print_message(__LINE__ . " ALMBaseline ̐ݒ: $baselinename");
	$record->SetFieldValue("ALMBuildType", $opts{buildtype});
		$logger->log_print_message(__LINE__ . " ALMBuildType ̐ݒ: $opts{buildtype}");
	$record->SetFieldValue("ALMOwner", $opts{user});
		$logger->log_print_message(__LINE__ . " ALMOwner ̐ݒ: $opts{user}");
	$record->SetFieldValue("ALMSecurityPolicy", $opts{securitypolicy});
		$logger->log_print_message(__LINE__ . " ALMSecurityPolicy ̐ݒ: $opts{securitypolicy}");	

	#Results from Building
	if ($opts{buildstatus}ne ""){
		$record->SetFieldValue("ALMBuildStatus", $opts{buildstatus});
		$logger->log_print_message(__LINE__ . " BuildStatus ̐ݒ: $opts{buildstatus}");
	}
	
	if ($opts{buildstarttime} ne ""){
		$record->SetFieldValue("Start_DateTime", $opts{buildstarttime});
		$logger->log_print_message(__LINE__ . " BuildStartTime ̐ݒ: $opts{buildstarttime}");
	}
	if ($opts{buildendtime} ne ""){
		$record->SetFieldValue("End_DateTime", $opts{buildendtime});
		$logger->log_print_message(__LINE__ . " BuildEndTime ̐ݒ: $opts{buildendtime}");
	}
		
	
	
	$logger->log_print_message(__LINE__ . " rh R[h $opts{id} ؂Ă܂");
	$status = "" ; #thl2
	eval {$status = $record->Validate()};
	if (($@) || ($status)) {  #thl2
		$logger->log_print_error(__LINE__ . " ClearQuest f[^x[XŃrh R[h؂ł܂ł: $status");
	}
	if ($@) {
		$record->Revert();
		$logger->log_print_error(__LINE__ . "main: ClearQuest f[^x[XŃrh R[h؂ł܂łB");
		$logger->log_print_error(__LINE__ . "main: $@");
                $logger->log_print_error(__LINE__ . "߂Xe[^X $status ł"); #thl2
		return 0;
	}
	$status = "" ; #thl2
	$logger->log_print_message(__LINE__ . " rh R[h $opts{id} R~bgĂ܂");
	eval {$status = $record->Commit()};
	if (($@) || ($status)) {  #thl2
		$record->Revert();
		$logger->log_print_error(__LINE__ . "main: ClearQuest f[^x[XŃrh R[hR~bgł܂łB");
		$logger->log_print_error(__LINE__ . "main: $@");
                $logger->log_print_error(__LINE__ . "߂Xe[^X $status ł"); #thl2
		CQPerlExt::CQSession_Unbuild($session);
		return 0;
	}
	$record = undef;
	$logger->log_print_success(__LINE__ . " $opts{id} ͐ɍXV܂B");
	CQPerlExt::CQSession_Unbuild($session);
	return 1;
	} #end block
$logger->log_print_success("Update_build_record XNvg܂B");
$logger->log_print_message(__LINE__ . " Update_build_record XNvg̊");
CQPerlExt::CQSession_Unbuild($session);
return 1;
} #end sub main
#### End of Script
#### SUBROUTINES FOR PROGRAM
